
angular.module('mobility-app').controller('BillingCtrl', function($scope, $timeout, $filter, $state, $ionicLoading, $ionicViewSwitcher, AppInfo, MenuService, backendData) {
	var self = this;

	//-- controller variables - accessed from DOM
	self.currentDate = 'Data loaded at ' + moment().format('MMMM DD, YYYY HH:mm:ss');

	self.data = $.extend(true, {}, backendData);
	self.billingCycles = [];

	//-- MENU CONTROL
	self.toggleSideMenu = function() {
		MenuService.toggle();
	};

	//-- Auto Refresh
	var promise = $timeout(function() {
		$state.go($state.current, {}, {reload: true});
	}, AppInfo.RefreshTime);

	//-- NAVIGATION
	self.goToChart = function(bcName) {
		$timeout.cancel(promise);
		$ionicLoading.show();
		window.cordova ? $ionicViewSwitcher.nextDirection("forward") : $ionicViewSwitcher.nextDirection("swap");
		$state.go('app.charts', { title: bcName, type: 'bill-cycle/' + bcName, previous: $state.current.name, groupBy: 'type' });
	};

	//-- INITIALIZATION
	function initializeBillingCycles() {
		var bcNames = [];
		var billCycles = [];

		var bc;
		for (var i=0; i < self.data.gauges.length; i++) {
			bc = self.data.gauges[i].title.split(' ')[0];
			if (bcNames.indexOf(bc) < 0) {
				bcNames.push(bc);
			}
		}

		for (var i=0; i < bcNames.length; i++) {
			billCycles[i] = {
				name: bcNames[i]
			};

			for (var j=0; j < self.data.gauges.length; j++) {
				bc = self.data.gauges[j];
				if (bc.title.indexOf(bcNames[i]) < 0) {
					continue;
				}

				if (bc.title.indexOf('Timeliness') > 0) {
					billCycles[i].timeliness = $filter('numberFormatter')(bc.metric.value.value, bc.metric.info.unit);
					billCycles[i].timelinessStatus = bc.metric.value.status;
				}
				else if (bc.title.indexOf('Accuracy') > 0) {
					billCycles[i].accuracy = $filter('numberFormatter')(bc.metric.value.value, bc.metric.info.unit, false);
					billCycles[i].accuracyStatus = bc.metric.value.status;
				}
				else if (bc.title.indexOf('Rejects') > 0) {
					billCycles[i].reject = $filter('gaugeNumberFormatter')(bc.metric.value.value, bc.metric.info.unit);
					billCycles[i].rejectStatus = bc.metric.value.status;
				}
			}

			for (var j=0; j < self.data.metrics.length; j++) {
				bc = self.data.metrics[j];
				if (bc.info.name.indexOf(bcNames[i]) < 0) {
					continue;
				}

				if (bc.info.name.indexOf('Adjustments Amount') > 0) {
					billCycles[i].adjustment = $filter('gaugeNumberFormatter')(bc.value.value, bc.info.unit);
					billCycles[i].adjustmentStatus = bc.value.status;
				}
				else if (bc.info.name.indexOf('Revenue RC') > 0) {
					billCycles[i].revenueRC = $filter('gaugeNumberFormatter')(bc.value.value, bc.info.unit);
					billCycles[i].revenueRCStatus = bc.value.status;
				}
				else if (bc.info.name.indexOf('Revenue OTC') > 0) {
					billCycles[i].revenueOTC = $filter('gaugeNumberFormatter')(bc.value.value, bc.info.unit);
					billCycles[i].revenueOTCStatus = bc.value.status;
				}
				else if (bc.info.name.indexOf('Supressed') > 0) {
					billCycles[i].supressed = $filter('gaugeNumberFormatter')(bc.value.value, bc.info.unit);
					billCycles[i].supressedStatus = bc.value.status;
				}
				else if (bc.info.name.indexOf('Bills') > 0) {
					billCycles[i].bills = $filter('gaugeNumberFormatter')(bc.value.value, bc.info.unit);
					billCycles[i].billsStatus = bc.value.status;
				}
				else if (bc.info.name.indexOf('Undo') > 0) {
					billCycles[i].undo = $filter('gaugeNumberFormatter')(bc.value.value, bc.info.unit);
					billCycles[i].undoStatus = bc.value.status;
				}
				else if (bc.info.name.indexOf('Cycle Status') > 0) {
					billCycles[i].cycleStatus = (bc.value.value == 0 ? 'In Progress' : 'Completed');
				}
			}
		}

		//sort bill cycles
		var after = false;
		var before = [];
		var arr = [];

		for (var i=0; i < billCycles.length; i++) {
			bc = billCycles[i];
			if (!after && bc.cycleStatus !== 'Completed') {
				after = true;
			}

			if (after) {
				arr.push(bc);
			} else {
				before.push(bc);
			}
		}

		return arr.concat(before);
	};

	//-- re-structure the data coming from backend
	$scope.$on('$ionicView.beforeEnter', function(event) {
		self.billingCycles = initializeBillingCycles();
	});

});
