
angular.module('mobility-app').controller('DashboardCtrl', function ($window, $scope, $filter, $timeout, $state, $ionicLoading, $ionicViewSwitcher, AppInfo, backendData, GaugeService, GraphService, MenuService, FileService) {
	var self = this;

	//-- local variables
	var graphSwiper;
	var barSwiper;

	//-- controller variables - accessed from DOM
	self.currentDate = 'Data loaded at ' + moment().format('MMMM DD, YYYY HH:mm:ss');

	self.data = $.extend(true, {}, backendData[0]);
	//self.data.topContributors = [];//$.extend(true, [], backendData[1].topContributors);

	self.graphs = null;
	self.callCenters = null;
	self.offers = null;
	self.gauges = null;

	//-- Auto Refresh
	$timeout(function () {
		$state.go($state.current, {}, { reload: true });
	}, AppInfo.RefreshTime);

	//--- MENU CONTROL
	self.toggleSideMenu = function () {
		MenuService.toggle();
	};

	//--- NAVIGATION
	function goToChart(title, type, index, groupBy) {
		$ionicLoading.show();
		window.cordova ? $ionicViewSwitcher.nextDirection("forward") : $ionicViewSwitcher.nextDirection("swap");
		$state.go('app.charts', { title: title, type: type, graphIndex: (index == undefined ? -1 : index), groupBy: groupBy });
	};

	self.goToCharts = function () {
		var _currentSelection = graphSwiper.activeIndex;
		switch (_currentSelection) {
			case 0: { // iPhone Activations
				//goToChart(self.graphs[_currentSelection].title, 'iphone-transactions');
				goToChart('iPhone Activations', 'iphone-transactions');
				break;
			}
			case 1: { // TLG TPM - not expending
				// goToChart(self.graphs[_currentSelection].title, 'tlg-tpm');
				break;
			}
			default:
				break;
		}


	};



	self.goToOrderFalloutCharts = function () {
		goToChart('Order Fallout', 'overview/order-fallout');
	};

	


	self.goToGaugeChart = function (gauge) {

		var _gaugeServiceNames = {
			'Activations': 'overview/activations', 
			'Upgrades and Data Change' : 'overview/upgrades',
			'Sim Changes' : 'overview/sim-changes',
			'Cancellations' : 'overview/cancellations',
			'Net Additions' : 'overview/net-additions'
		};
		goToChart(gauge.title, _gaugeServiceNames[gauge.title]);
	};

	self.openReport = function (report) {
		if (self.isReportAvailable(report)) {
			$ionicLoading.show();
			window.cordova ? $ionicViewSwitcher.nextDirection("forward") : $ionicViewSwitcher.nextDirection("swap");
			$state.go('app.report', { report: report });
		}
	};

	self.isReportAvailable = function (report) {
		return true;//self.data.isMornAvailEnabled;
	};



	// MESSAGE CENTER
	self.alertMessage = {};
	self.showMessageDetails = false;

	self.goToMessage = function (messageId) {
		self.alertMessage = {};
		for (var i = 0; i < self.data.issues.length; i++) {
			if (self.data.issues[i].id == messageId) {
				self.alertMessage = self.data.issues[i];
				break;
			}
		}
		self.showMessageDetails = !self.showMessageDetails;
	};





	//--- INITIALIZATION
	function initializeGraphs() {
		return [GraphService.getGraphDataByTitle('All iPhones Activations Hourly', self.data.graphs),
		GraphService.getGraphDataByTitle('TLG Transactions per Minute', self.data.graphs)
		];
	};

	function initializeGauges() {
		return [
			GaugeService.getGaugeByTitle('Activations', self.data.gauges),
			GaugeService.getGaugeByTitle('Upgrades and Data Change', self.data.gauges),
			GaugeService.getGaugeByTitle('Sim Changes', self.data.gauges),
			GaugeService.getGaugeByTitle('Cancellations', self.data.gauges),
			GaugeService.getGaugeByTitle('Net Additions', self.data.gauges)];
	};

	// function initializeOffers() {
	// 	var arr = self.data.topContributors || [{}];
	// 	if (arr[0]) {
	// 		arr[0].title = "Top Devices";
	// 		if (arr[0].contributors) {
	// 			if (arr[0].contributors.length > 5) {
	// 				arr[0].contributors = arr[0].contributors.slice(0, 5);
	// 			}
				
	// 		} else {
	// 			arr[0].contributors = [];
	// 		}

	// 		// arr[1].title = "Top Product Offers";
	// 		// if (arr[1].contributors && arr[1].contributors.length > 5) {
	// 		// 	arr[1].contributors = arr[1].contributors.slice(0, 5);
	// 		// }

	// 		return [arr[0]];	
	// 	}

	// 	return [[]];
	// };



	//-- re-structure the data coming from backend
	$scope.$on('$ionicView.beforeEnter', function (event) {
		self.graphs = initializeGraphs();
		//self.orders = initializeOrderBars();
		//self.usageGauges = initializeUsageGauges();

		//self.offers = initializeOffers();
		self.gauges = initializeGauges();

	});

	//-- draw gauges and graphs after elements are built
	$scope.$on('$ionicView.afterEnter', function (event) {
		//-- redraw gauges and graph on window resize  
		if (!window.cordova) {
			angular.element($window).bind('resize', function () {
				// wrap around a timeout to wait elements resizing before drawing gauges and graphs
				$timeout(function () {
					initialize(true);
				}, 300);
			});
		}

		// a timeout zero, puts the function in next digest cycle.
		$timeout(function () {
			initialize(false);
		}, 0);
	});

	function initialize(isUpdate) {
		if (isUpdate == true) {
			graphSwiper.update();
			graphSwiper.slideTo(graphSwiper.activeIndex);

			// barSwiper.update();
			// barSwiper.slideTo(barSwiper.activeIndex);
		}
		else {
			graphSwiper = new Swiper('.graph .swiper-container', {
				pagination: '.graph .swiper-container .swiper-pagination',
				paginationClickable: true,
				spaceBetween: 80,
				// desktop
				mousewheelControl: true,
				// events
				onSlideChangeEnd: function (swiper) {
					// notify swipe changed event
					$scope.$broadcast('graphSwiperSlideDidChange', [swiper.activeIndex]);
				}

			});

			// barSwiper = new Swiper('.offers .swiper-container', {
			// 	pagination: '.offers .swiper-container .swiper-pagination',
			// 	paginationClickable: true,
			// 	spaceBetween: 80,
			// 	// desktop
			// 	mousewheelControl: true
			// });
		}

		drawGraphs();
		//	drawOrderBars();
		//drawOfferBars();
		drawGauges();

		registerToGraphSlideEvent();


	};


	function registerToGraphSlideEvent() {
		var _listener = $scope.$on('graphSwiperSlideDidChange', function (event, data) {
			if (data == 0) {
				angular.element(document.getElementById('graph-expand-btn')).removeClass('disabled');
			} else {
				angular.element(document.getElementById('graph-expand-btn')).addClass('disabled');
			}
		});

		// Unregister
		$scope.$on('$destroy', function () {
			_listener();
		});

	};




	//-- GRAPHS
	function drawGraphs() {
		for (var i = 0; i < self.graphs.length; i++) {
			GraphService.drawGraph('graph' + i, self.graphs[i]);
		}
	};

	//-- BAR GRAPH
	function drawOfferBars() {
		for (var i = 0; i < self.offers.length; i++) {

			if (self.offers[i].contributors.length > 0) {
				GraphService.drawBarGraph('offer-chart' + i, self.offers[i]);
			}


		}
	};

	//-- BARS
	function drawCallCenterBars() {
		if (!self.callCenters) return;

		for (var i = 0; i < self.callCenters.length; i++) {
			GaugeService.drawHorizontalGauge(document.getElementById('metric' + i), self.callCenters[i]);
		}
	};

	//-- GAUGES
	function drawGauges() {
		for (var i = 0; i < self.gauges.length; i++) {
			GaugeService.draw('gauge' + i, self.gauges[i], 'arrow');
		}
	};

});
