
angular.module('mobility-app').controller('OperationsCtrl', function($scope, $window, $timeout, $state, $ionicLoading, $ionicViewSwitcher, AppInfo, MenuService, GraphService, GaugeService, backendData) {
	var self = this;

	//-- local variables
	var graphSwiper;

	//-- controller variables - accessed from DOM
	self.currentDate = 'Data loaded at ' + moment().format('MMMM DD, YYYY HH:mm:ss');

	self.data = $.extend(true, {}, backendData);
	self.graphs = null;
	self.gauges = null;
	self.tickets = null;
	self.bars = null;

	//-- MENU CONTROL
	self.toggleSideMenu = function() {
		MenuService.toggle();
	};

	//-- Auto Refresh
	var promise = $timeout(function() {
		$state.go($state.current, {}, {reload: true});
	}, AppInfo.RefreshTime);

	//-- NAVIGATION
	function goToChart(title, type, index) {
		$timeout.cancel(promise);
		$ionicLoading.show();
		window.cordova ? $ionicViewSwitcher.nextDirection("forward") : $ionicViewSwitcher.nextDirection("swap");
		$state.go('app.charts', { title: title, type: type, graphIndex: (index == undefined ? -1 : index), previous: $state.current.name});
	};

	self.goToCharts = function() {
		goToChart(self.graphs[graphSwiper.activeIndex].title, 'operations/tickets-backlog', graphSwiper.activeIndex);
	};

	self.goToDiscrepancies = function() {
		goToChart('Discrepancies', 'operations/data-discrepancies');
	};

	self.goToGaugeChart = function(gauge) {
		var type = (gauge.title.indexOf('BAP') >= 0) ? 'operations/crm-bap-backlong' : 'operations/trb-backlog';
		goToChart(gauge.title, type);
	};

	//-- INITIALIZATION
	function initializeGraphs() {

		var graphs = [];

		if (!self.data.graphs || self.data.graphs.length < 1) return self.data.graphs;

		graphs.push(self.data.graphs[0]);
		graphs.push(self.data.graphs[1]);

		var sum = {
			title: "Ticket Incoming - Backlog",
			metricSeries: []
		};

		var severity = "";
		var sumValue = 0;
		for (var i=0; i < self.data.graphs[0].metricSeries.length; i++) {
			severity = 'P' + (i+1);
			sum.metricSeries.push({
				info: {
					id: 0,
					name: severity + ' Backlog',
					unit: self.data.graphs[0].metricSeries[i].info.unit,
					updateSchedule: self.data.graphs[0].metricSeries[i].info.updateSchedule
				},
				values: []
			});

			for (var j=0; j < self.data.graphs[0].metricSeries[i].values.length; j++) {
				sumValue = self.data.graphs[0].metricSeries[i].values[j].value;
				for (var k=0; k < self.data.graphs[1].metricSeries.length; k++) {
					if (self.data.graphs[1].metricSeries[k].info.name.indexOf(severity) >= 0) {
						sumValue += self.data.graphs[1].metricSeries[k].values[j].value;
						break;
					}
				}

				sum.metricSeries[i].values.push({
					date: self.data.graphs[0].metricSeries[i].values[j].date,
					value: sumValue,
					status: self.data.graphs[0].metricSeries[i].values[j].status
				});
			}
		}

		graphs.push(sum);

		graphs.push(self.data.graphs[2]);

		return graphs;
	};

	function initializeGauges() {
		return [GaugeService.getGaugeByTitle('Error Backlog', self.data.gauges), GaugeService.getGaugeByTitle('CRM BAP Backlog', self.data.gauges)];
	};

	function initializeTickets() {
		return [GaugeService.getGaugeByTitle('P1 Tickets', self.data.gauges), GaugeService.getGaugeByTitle('P2 Tickets', self.data.gauges)];
	};

	function initializeBars() {
		var bars = [{
			title: 'Barred in Ordering & Switch',
			gauges: [GaugeService.getGaugeByTitle('Barred in Ordering & Switch - Cancelled in Collections', self.data.gauges), GaugeService.getGaugeByTitle('Barred in Ordering & Switch - Working in Collections', self.data.gauges)]
		}, {			
			title: 'Working in Ordering & Switch',
			gauges: [GaugeService.getGaugeByTitle('Working in Ordering & Switch - Cancelled in Collections', self.data.gauges), GaugeService.getGaugeByTitle('Working in Ordering & Switch - Barred in Collections', self.data.gauges)]
		}];

		return bars;
	};

	//-- re-structure the data coming from backend
	$scope.$on('$ionicView.beforeEnter', function(event) {
		self.graphs = initializeGraphs();
		self.gauges = initializeGauges();
		self.tickets = initializeTickets();
		self.bars = initializeBars();
	});

	//-- draw gauges and graphs after elements are built
	$scope.$on('$ionicView.afterEnter', function(event) {
		//-- redraw gauges and graph on window resize  
		if (!window.cordova) {
			angular.element($window).bind('resize', function () {
				// wrap around a timeout to wait elements resizing before drawing gauges and graphs
				$timeout(function() {
					initialize(true);
				}, 300);
			});
		}

		// a timeout zero, puts the function in next digest cycle.
		$timeout(function() {
			initialize(false);
		}, 0);
	});

	function initialize(isUpdate) {
		if (isUpdate == true) {
			graphSwiper.update();
			graphSwiper.slideTo(graphSwiper.activeIndex);
		}
		else {
			graphSwiper = new Swiper('.graph .swiper-container', {
				pagination: '.graph .swiper-container .swiper-pagination',
				paginationClickable: true,
				spaceBetween: 80,
				// desktop
				mousewheelControl: true
		    });
		}

		drawGraphs();
		drawBars();
		drawGauges();
	};

	//-- GRAPHS
	function drawGraphs() {
		for (var i=0; i < self.graphs.length; i++) {
			if (!self.graphs[i].metricSeries || self.graphs[i].metricSeries.length < 1) {
				continue;
			}

			GraphService.drawGraph('graph' + i, self.graphs[i]);
		}
	};

	//-- GAUGES
	function drawGauges() {
		for (var i=0; i < self.gauges.length; i++) {
			GaugeService.draw('gauge' + i, self.gauges[i], 'arrow');
		}
	};

	//-- BARS
	function drawBars() {
		if (!self.bars) return;

		for (var i=0; i < self.bars.length; i++) {
			for (var j=0; j < self.bars[i].gauges.length; j++) {
				GaugeService.drawHorizontalGauge(document.getElementById('bar' + ((i*2) + j)), self.bars[i].gauges[j]);
			}
		}
	};

});
