
angular.module('mobility-app').controller('PdfCtrl', function($scope, $ionicLoading, $ionicViewSwitcher, $ionicHistory, pdfFile) {
	var self = this;

    var canvas = document.getElementById('pdfCanvas');
    var ctx = canvas.getContext('2d');
	var pdfDoc = null;
    var pageNum = 0;
    var pageRendering = false;

    self.zoom = 0;

    PDFJS.disableWorker = true;

	// NAVIGATION
	self.goBack = function(event) {
		event.stopPropagation();

		$ionicLoading.show();
		$ionicViewSwitcher.nextDirection("back");
		$ionicHistory.goBack();
	};

	$scope.$on('$ionicView.afterEnter', function() {
		PDFJS.getDocument(pdfFile).then(function(_pdfDoc) {
		    pdfDoc = _pdfDoc;
		    pageNum = 1;
		    self.zoom = 100;
		    renderPage();
		    $scope.$apply();
		});
	});

	self.canZoomIn = function() {
		if (self.zoom <= 10) {
			return false;
		}
		return true;
	};

	self.zoomIn = function() {
		if (pageRendering) return;
		if(self.canZoomIn()) {
			self.zoom -= 10;
			renderPage();
		}
	};

	self.canZoomOut = function() {
		if (self.zoom >= 190) {
			return false;
		}
		return true;
	};

	self.zoomOut = function() {
		if (pageRendering) return;
		if (self.canZoomOut()) {
			self.zoom += 10;
			renderPage();
		}
	};

	self.hasPrevPage = function() {
		return (pageNum > 1);
	};

	self.hasNextPage = function() {
		if (pdfDoc == null) {
			return 0;
		}
		return (pageNum < pdfDoc.numPages);
	};

	self.getPageNumber = function() {
		if (pdfDoc == null) {
			return pageNum;
		}
		return pageNum + ' / ' + pdfDoc.numPages;
	};
	
	self.previousPage = function () {
		if (pageRendering) return;
	    if (pageNum <= 1) return;
	    pageNum--;
	    renderPage();
	}

	self.nextPage = function() {
		if (pageRendering) return;
	    if (pageNum >= pdfDoc.numPages) return;
	    pageNum++;
	    renderPage();
	}

	function renderPage() {
	    // Using promise to fetch the page
		pageRendering = true;
	    pdfDoc.getPage(pageNum).then(function(page) {
	    	var scale = 1;
	    	if (self.zoom != 100) {
	    		scale = self.zoom < 100 ? 1 + (100 - self.zoom)/100 : 1 - (self.zoom - 100)/100;
	    		scale = Number(scale.toFixed(2));
	    	}

	        var maxWidth = canvas.parentElement.clientWidth;
	        var maxHeight = canvas.parentElement.clientHeight;

	        var viewport = page.getViewport(1.0); // unscaled
	        if (viewport.height >= viewport.width) {
		        viewport = page.getViewport(maxHeight / page.getViewport(scale).height);
	        }
	        else {
		        viewport = page.getViewport(maxWidth / page.getViewport(scale).width);
	        }

        	canvas.width = viewport.width;
        	canvas.height = viewport.height;

        	// Render PDF page into canvas context
	        var renderContext = {
	            canvasContext: ctx,
	            viewport: viewport
	        };

	        page.render(renderContext).promise.then(function() {
	        	pageRendering = false;
	        	$scope.$apply();
	        });
	    });
	}
});
