
angular.module('mobility-app').controller('RevenueCtrl', function($scope, $window, $timeout, $state, $ionicLoading, $ionicViewSwitcher, $filter, AppInfo, MenuService, GraphService, GaugeService, backendData) {
	var self = this;

	var graphSwiper = null;

	//-- controller variables - accessed from DOM
	self.currentDate = 'Data loaded at ' + moment().format('MMMM DD, YYYY HH:mm:ss');

	self.data = $.extend(true, {}, backendData);
	self.graphs = [];
	self.gauges = [];
	self.rates = [];

	//-- MENU CONTROL
	self.toggleSideMenu = function() {
		MenuService.toggle();
	};

	//-- Auto Refresh
	var promise = $timeout(function() {
		$state.go($state.current, {}, {reload: true});
	}, AppInfo.RefreshTime);

	
	//-- NAVIGATION
	function goToChart(title, type, index) {
		console.log('goToChart ' + title + ': ' + type);
		$timeout.cancel(promise);
		$ionicLoading.show();
		window.cordova ? $ionicViewSwitcher.nextDirection("forward") : $ionicViewSwitcher.nextDirection("swap");
		$state.go('app.charts', { title: title, type: type, graphIndex: (index == undefined ? -1 : index), previous: $state.current.name});
		
	};

	self.goToCharts = function() {
		goToChart(self.graphs[graphSwiper.activeIndex].title, 'revenue', graphSwiper.activeIndex);
	};

	self.goToCollectionCharts = function() {
		goToChart('Collection', 'revenue/collection');
	};

	self.goToSuccessRateCharts = function() {
		goToChart('Success Rate', 'revenue/success-rates');
	};

	self.goToGaugeChart = function(gauge) {
		var type = (gauge.title.indexOf('Recharge') >= 0) ? 'revenue/recharge-sucess-rate' : (gauge.title.indexOf('Prepaid') >= 0) ? 'revenue/prepaid-latency' : 'revenue/postpaid-latency';
		goToChart(gauge.title, type);
	};

	//-- INITIALIZATION
	function initializeGraphs() {
		var graphs = self.data.graphs;

		for (var i=0; i < graphs.length; i++) {
			for (var j=0; j < graphs[i].metricSeries.length; j++) {
				graphs[i].metricSeries[j].info.name = graphs[i].metricSeries[j].info.name.replace('Postpaid - ', '').replace('Prepaid - ', '');
			}
		}

		return graphs;
	};

	function initializePieChart() {
		var data = [];
		var name;

		for (var i=0; i < self.data.metrics.length; i++) {
			name = self.data.metrics[i].info.name;
			name = name.substring(name.indexOf('(')+1, name.length-1);
			
			
			data.push({
				name: name,
				value: self.data.metrics[i].value.value
			})
		}

		return data;
	};

	function initializeGauges() {
		return [GaugeService.getGaugeByTitle('Prepaid Recharge Success Rate', self.data.gauges)];
	};

	function initializeLatencyGauges() {
		var latencyPre = GaugeService.getGaugeByTitle('Avg Latency - Prepaid', self.data.gauges);
		latencyPre.details = [GaugeService.getGaugeByTitle('Prepaid Latency - SMS', self.data.gauges), GaugeService.getGaugeByTitle('Prepaid Latency - MMS', self.data.gauges), GaugeService.getGaugeByTitle('Prepaid Latency - Voice', self.data.gauges), GaugeService.getGaugeByTitle('Prepaid Latency - GPRS', self.data.gauges), GaugeService.getGaugeByTitle('Prepaid Latency - Content', self.data.gauges)];

		var latencyPost = GaugeService.getGaugeByTitle('Avg Latency - Postpaid', self.data.gauges);
		latencyPost.details = [GaugeService.getGaugeByTitle('Postpaid Latency - SMS', self.data.gauges), GaugeService.getGaugeByTitle('Postpaid Latency - MMS', self.data.gauges), GaugeService.getGaugeByTitle('Postpaid Latency - Voice', self.data.gauges), GaugeService.getGaugeByTitle('Postpaid Latency - GPRS', self.data.gauges), GaugeService.getGaugeByTitle('Postpaid Latency - Content', self.data.gauges)];

		return [latencyPre, latencyPost];
	};

	function initializeRates() {
		var titles = ['Turbo Charging', 'Customer MNG', 'Replenishment', 'EDR'];
		var rates = [];

		var prepaid, postpaid;
		for (var i=0; i < titles.length; i++) {
			prepaid = GaugeService.getGaugeByTitle('Success Rate Prepaid - ' + titles[i], self.data.gauges);
			postpaid = GaugeService.getGaugeByTitle('Success Rate Postpaid - ' + titles[i], self.data.gauges);
			if (prepaid.metric.value.value == null)
				prepaid.metric.value.value = 'NA';
			if (postpaid.metric.value.value == null)
				postpaid.metric.value.value = 'NA';
			
			rates.push({
				name: titles[i],
				prepaid: prepaid.metric.value.value + '%',
				prepaidStatus: prepaid.metric.value.status,
				postpaid: postpaid.metric.value.value + '%',
				postpaidStatus: postpaid.metric.value.status
			});
		}

		return rates;
	};

	$scope.$on('$ionicView.beforeEnter', function(event) {
		self.graphs = initializeGraphs();
		self.gauges = initializeGauges();
		self.latency = initializeLatencyGauges();
		self.collection = initializePieChart();
		self.rates = initializeRates();
	});

	//-- draw gauges and graphs after elements are built
	$scope.$on('$ionicView.afterEnter', function(event) {
		//-- redraw gauges and graph on window resize  
		if (!window.cordova) {
			angular.element($window).bind('resize', function () {
				// wrap around a timeout to wait elements resizing before drawing gauges and graphs
				$timeout(function() {
					initialize(true);
				}, 300);
			});
		}

		// a timeout zero, puts the function in next digest cycle.
		$timeout(function() {
			initialize(false);
		}, 0);
	});

	function initialize(isUpdate) {
		if (isUpdate == true) {
			graphSwiper.update();
			graphSwiper.slideTo(graphSwiper.activeIndex);
		}
		else {
			graphSwiper = new Swiper('.graph .swiper-container', {
				pagination: '.graph .swiper-container .swiper-pagination',
				paginationClickable: true,
				spaceBetween: 80,
				// desktop
				mousewheelControl: true
		    });
		}

		drawGraphs();
		drawPieChart();
		drawGauges();
	};

	//-- GRAPHS
	function drawGraphs() {
		for (var i=0; i < self.graphs.length; i++) {
			GraphService.drawGraph('graph' + i, self.graphs[i]);
		}
	};

	//-- PIE CHART
	function drawPieChart() {
		GraphService.drawPieChart('collection-chart', self.collection);
	};

	//-- GAUGES
	function drawGauges() {
		for (var i=0; i < self.gauges.length; i++) {
			GaugeService.draw('gauge' + i, self.gauges[i], 'arrow');
		}
		for (var i=0; i < self.latency.length; i++) {
			GaugeService.draw('latency' + i, self.latency[i], 'arrow');
		}
	};

});
