
var app = angular.module('mobility-app');
app.filter('dateFormatter', dateFormatterFilter);
app.filter('legendTitle', legendTitleFilter);
app.filter('numberFormatter', numberFormatterFilter);
app.filter('graphNumberFormatter', graphNumberFormatterFilter);
app.filter('gaugeNumberFormatter', gaugeNumberFormatterFilter);
app.filter('translate', translateFilter);
app.filter('convertMinutes', convertMinutesFilter);
app.filter('convertHours', convertHoursFilter);
app.filter('convertDay', convertDayFilter);
app.filter('convertDate', convertDateFilter);

function convertDateFilter() {
	return function (strDate) {
		//debugger;
		var myScreen = $(window).width();
		var tmp = strDate.substring(15);
		if (myScreen <= 768) {
			return moment(tmp, 'MMMM DD, YYYY  HH:mm:ss').format('DD MMM, YYYY HH:mm:ss');
		}

		return strDate;
	}
};

function dateFormatterFilter() {
	return function (strDate, format) {
		if (!strDate) {
			return '';
		}
		var out = format ? format : 'DD MMM';
		return moment(strDate.slice(0, 19), 'YYYY-MM-DDTHH:mm:ss').format(out);
	}
};

function legendTitleFilter() {
	return function (str) {
		if (!str) {
			return '';
		}

		return str.split('|')[2];
	}
};

function numberFormatterFilter($filter) {
	return function (number, unit, fractionSize) {

		if (isNaN(number)) return number;

		if (unit == 'MINUTES') return $filter('convertMinutes')(number);

		if (unit == 'HOURS') return $filter('convertHours')(number);

		var preSymbol = '';
		var symbol = '';
		var value = Number(number);
		var formatted = value;

		var decimals = fractionSize ? fractionSize : 2;

		if (unit == 'MILLISECONDS') {
			symbol = ' ms';
			decimals = 0;
		}

		else if (value >= 1000000000000) {
			formatted = value / 1000000000000;
			symbol = 't';
		}
		else if (value >= 1000000000) {
			formatted = value / 1000000000;
			symbol = 'b';
		}
		else if (value >= 1000000) {
			formatted = value / 1000000;
			symbol = 'm';
		}
		else if (value >= 1000) {
			formatted = value / 1000;
			symbol = 'k';
		}

		if (unit == 'PERCENTAGE') {
			symbol = '%';
		}

		if (unit == 'DOLLAR') {
			preSymbol = '$';
		}

		return preSymbol + $filter('number')(formatted, decimals) + symbol;
	}
};

function graphNumberFormatterFilter($filter) {
	return function (value, unit, truncate) {

		//return $filter('number')(value);

		if (isNaN(value)) return value;

		if (unit == 'MINUTES') return $filter('convertMinutes')(value);

		if (unit == 'HOURS') return $filter('convertHours')(value);

		var preSymbol = '';
		var symbol = '';
		var decimals = 0;

		value = Number(value);
		var formatted = value;

		if (unit == 'MILLISECONDS') {
			symbol = ' ms';
		}

		else if (value >= 1000000000000) {
			formatted = value / 1000000000000;
			symbol = 't';
		}
		else if (value >= 1000000000) {
			formatted = value / 1000000000;
			symbol = 'b';
		}
		else if (value >= 1000000) {
			formatted = value / 1000000;
			symbol = 'm';
		}
		else if (value >= 1000) {
			formatted = value / 1000;
			symbol = 'k';
		}

		if (!truncate) {
			if (formatted >= 100) {
				decimals = 0;
			}
			else if (formatted >= 10) {
				decimals = 1;
				formatted = Math.floor(formatted * 10) / 10;
			}
			else {
				decimals = 2;
				formatted = Math.floor(formatted * 100) / 100;
			}
		}

		if (unit == 'PERCENTAGE') {
			symbol = '%';
		}

		if (unit == 'DOLLAR') {
			preSymbol = '$';
		}

		var str = preSymbol + formatted.toFixed(decimals) + symbol;
		if (str.indexOf('.000' + symbol) >= 0) {
			str = preSymbol + formatted.toFixed(0) + symbol;
		}
		return str;
	}
};

function gaugeNumberFormatterFilter($filter) {
	return function (value, unit, truncate) {

		if (isNaN(value)) return value;

		if (unit == 'MINUTES') return $filter('convertMinutes')(value);

		if (unit == 'HOURS') return $filter('convertHours')(value);

		if (unit == 'DAY') return $filter('convertDay')(value);

		var signal = '';
		var preSymbol = '';
		var symbol = '';
		var decimals = 0;

		value = Number(value);

		if (value < 0) {
			signal = '- ';
			value = value * (-1);
		}

		var formatted = value;

		if (unit == 'MILLISECONDS') {
			symbol = ' ms';
		}

		else if (value >= 1000000000000) {
			formatted = value / 1000000000000;
			symbol = 't';
		}
		else if (value >= 1000000000) {
			formatted = value / 1000000000;
			symbol = 'b';
		}
		else if (value >= 1000000) {
			formatted = value / 1000000;
			symbol = 'm';
		}
		else if (value >= 1000) {
			formatted = value / 1000;
			symbol = 'k';
		}

		if (!truncate) {
			if (formatted >= 100) {
				decimals = 0;
			}
			else if (formatted >= 10) {
				decimals = 1;
				formatted = Math.floor(formatted * 10) / 10;
			}
			else {
				decimals = 2;
				formatted = Math.floor(formatted * 100) / 100;
			}
		}

		if (unit == 'PERCENTAGE') {
			symbol = '%';
		}

		if (unit == 'DOLLAR') {
			preSymbol = '$';
		}

		return signal + preSymbol + formatted.toFixed(decimals) + symbol;
	}
};

function translateFilter() {
	return function (str) {
		if (!str) return '';
		if (String(str).toLowerCase() == 'bridge') {
			return 'puente';
		}
		return str;
	}
};

function convertMinutesFilter() {
	return function (value) {

		if (isNaN(value)) return value;

		var hour = Math.floor(value / 60);

		var min = Math.round(value - (hour * 60));

		// if (min < 10) {
		// 	min = '0' + min;
		// }


		if (hour == 0) {
			return min + 'm';
		}

		return hour + 'h ' + min + 'm';
	}
};

function convertHoursFilter() {
	return function (value) {

		if (isNaN(value)) return value;

		var hour = Math.floor(value);
		var min = Math.floor((value - hour) * 60);

		if (min < 10) {
			min = '0' + min;
		}

		return hour + 'h ' + min + 'm';
	}
};

function convertDayFilter() {
	return function (value) {
		if (isNaN(value)) return value;
		return moment(new Date(value)).format('DD/MM/YY HH:mm:ss');
	}
};


// IE Polyfill
// https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/String/endsWith

if (!String.prototype.endsWith) {
	String.prototype.endsWith = function (searchString, position) {
		var subjectString = this.toString();
		if (typeof position !== 'number' || !isFinite(position) || Math.floor(position) !== position || position > subjectString.length) {
			position = subjectString.length;
		}
		position -= searchString.length;
		var lastIndex = subjectString.lastIndexOf(searchString, position);
		return lastIndex !== -1 && lastIndex === position;
	};
}
