
var app = angular.module('mobility-app');
app.service('BackendResponseHandling', backendResponseHandling);
app.service('BackendService', backendService);



function backendResponseHandling($ionicLoading, $state, $localstorage) {
	var self = this;

	self.handle = function (response) {
		if (response.status == 200) {
			if (response.data.status) {
				if (response.data.status == 200) {
					return response.data.data;
				}
				else {
					return self.handleError(response.data);
				}
			}
			else {
				return response.data;
			}
		}
		else {
			return self.handleError(response);
		}
	};

	self.handleError = function (error) {
		$ionicLoading.hide();

		if (error.message) {
			alert(error.message);
		}
		else if (error.status == 401 || error.status == 403) {
			//alert(error.status + ' - The session has expired. Please login.');
			alert(error.data.message);
			$state.go('login');
		}
		else {
			alert(JSON.stringify(error));
		}

		return;
	};
};

function backendService($http, $q, $state, AppInfo, BackendResponseHandling, AuthService) {
	var self = this;

	self.getUserProfile = function () {
		if (AppInfo.useMockedData) {
			var defer = $q.defer();
			defer.resolve(MockedData['user'].data);
			return defer.promise;
		}

		var request = {
			method: 'GET',
			url: '/users/profiles'
		};
		return call(request);
	};

	self.getDashboardData = function () {

		if (AppInfo.useMockedData) {
			var defer = $q.defer();
			//defer.resolve([MockedData['overview'].data, MockedData['top-product-devices'].data]);
			defer.resolve([MockedData['overview'].data]);



			return defer.promise;
		}

		// var urls = ['/dashboard/views?viewType=overview',
		// 	'/dashboard/views?viewType=top-product-devices'];



		var urls = ['/dashboard/views?viewType=overview'];


		return getMultiple(urls);
	};

	self.getBillingData = function () {
		if (AppInfo.useMockedData) {
			var defer = $q.defer();
			defer.resolve(MockedData['bill-cycles'].data);
			return defer.promise;
		}

		var request = {
			method: 'GET',
			url: '/dashboard/views?viewType=bill-cycles'
		};
		return call(request);
	};


	self.getTlgTpmGraphs = function () {
		if (AppInfo.useMockedData) {
			var defer = $q.defer();
			defer.resolve([MockedData['overview/tlg-tpm'].data]);
			return defer.promise;
		}


		var urls = ['/dashboard/views?viewType=overview/tlg-tpm'];
		return getMultiple(urls);
	};


	self.getReport = function () {

		if (AppInfo.useMockedData) {
			var defer = $q.defer();
			var localRequest = {
				method: 'GET',
				url: 'templates/mockReport.html'
			};

			$http(localRequest).then(
				function (response) {
					defer.resolve(response.data);
				},
				function (error) {
					console.log('Error: ' + JSON.stringify(error));
					defer.reject(error);
				}
			);
			return defer.promise;
		}

		var request = {
			method: 'GET',
			url: '/downloads/Report.html'
		};

		return call(request);
	};











	self.getBillCycleGraph = function (type) {
		if (AppInfo.useMockedData) {
			var defer = $q.defer();
			defer.resolve([MockedData[type].data]);
			return defer.promise;
		}

		return getMultiple(['/dashboard/views?viewType=' + type]);

	};

	self.getRevenueData = function () {
		if (AppInfo.useMockedData) {
			var defer = $q.defer();
			defer.resolve(MockedData['revenue'].data);
			return defer.promise;
		}

		var request = {
			method: 'GET',
			url: '/dashboard/views?viewType=revenue'
		};
		return call(request);
	};

	self.getOperationsData = function () {
		if (AppInfo.useMockedData) {
			var defer = $q.defer();
			defer.resolve(MockedData['operations'].data);
			return defer.promise;
		}

		var request = {
			method: 'GET',
			url: '/dashboard/views?viewType=operations'
		};
		return call(request);
	};

	self.getSystemHealthData = function () {
		if (AppInfo.useMockedData) {
			var defer = $q.defer();
			defer.resolve(MockedData['system-health'].data);
			return defer.promise;
		}

		var request = {
			method: 'GET',
			url: '/dashboard/views?viewType=system-health'
		};
		return call(request);
	};

	self.getGlossaryData = function () {

		if (AppInfo.useMockedData) {
			var defer = $q.defer();
			defer.resolve(MockedData['glossary'].data);
			return defer.promise;
		}


		var defer = $q.defer();
		defer.resolve({});
		return defer.promise;


		// @TODO: check last update and use cached if available

		//var lastUpdated = $localstorage.getLastGlossaryLoadDate();
		// var request = {
		// 	method: 'GET',
		// 	url: '/glossary?timestamp='+lastUpdated
		// };
		// return call(request);



	};

	self.getWeekMonthGraphs = function (type) {
		if (AppInfo.useMockedData) {
			var defer = $q.defer();
			defer.resolve([MockedData[type + '/weekly'].data, MockedData[type + '/monthly'].data]);
			return defer.promise;
		}

		var urls = ['/dashboard/views?viewType=' + type + '/weekly',
		'/dashboard/views?viewType=' + type + '/monthly'];

		return getMultiple(urls);
	};

	

	self.getWeekMonthQuarterGraphs = function (type) {
		if (type == 'overview') {
			return getMultipleViews(['overview/total-orders', 'overview/total-subscribers', 'overview/total-recharge-transactions', 'overview/total-recharge-revenue', 'overview/churn']);
		}

		if (type == 'revenue') {
			return getMultipleViews(['revenue/postpaid-trends', 'revenue/prepaid-trends']);
		}

		if (AppInfo.useMockedData) {
			var defer = $q.defer();
			defer.resolve([MockedData[type + '/weekly'].data, MockedData[type + '/monthly'].data, MockedData[type + '/quarterly'].data]);
			return defer.promise;
		}

		var urls = ['/dashboard/views?viewType=' + type + '/weekly',
		'/dashboard/views?viewType=' + type + '/monthly',
		'/dashboard/views?viewType=' + type + '/quarterly'];

		return getMultiple(urls);
	};

	self.getPasswordPolicies = function () {
		var defer = $q.defer();

		var request = {
			method: 'GET',
			url: AppInfo.RegisterUserURL + 'policies/passwordPolicies?date=' + new Date().getTime(),
			contentType: 'application/json'
		};

		$http(request).then(
			function (response) {
				var handledResponse = BackendResponseHandling.handle(response);
				if (handledResponse) {
					defer.resolve(handledResponse);
				}
				else {
					defer.reject();
				}
			},
			function (error) {
				console.log('Error: ' + JSON.stringify(error));
				BackendResponseHandling.handleError(error);
				defer.reject(error);
			}
		);

		return defer.promise;
	};

	self.submitUserData = function (data) {
		var request = {
			method: 'POST',
			url: AppInfo.RegisterUserURL + 'users?date=' + new Date().getTime(),
			contentType: 'application/json',
			data: JSON.stringify(data)
		};

		return $http(request);
	};

	self.submitPasswordReset = function (data) {
		var request = {
			method: 'POST',
			url: AppInfo.RegisterUserURL + 'users/password?date=' + new Date().getTime(),
			contentType: 'application/json',
			data: JSON.stringify(data)
		};

		return $http(request);
	};


	function getMultipleViews(views) {
		if (AppInfo.useMockedData) {
			var defer = $q.defer();
			var responses = [];
			for (var i = 0; i < views.length; i++) {
				responses.push(MockedData[views[i] + '/weekly'].data);
				responses.push(MockedData[views[i] + '/monthly'].data);
				responses.push(MockedData[views[i] + '/quarterly'].data);
			}

			defer.resolve(responses);
			return defer.promise;
		}

		var urls = [];
		for (var i = 0; i < views.length; i++) {
			urls.push('/dashboard/views?viewType=' + views[i] + '/weekly');
			urls.push('/dashboard/views?viewType=' + views[i] + '/monthly');
			urls.push('/dashboard/views?viewType=' + views[i] + '/quarterly');
		}

		return getMultiple(urls);
	};


	function getMultiple(urls) {
		var defer = $q.defer();

		var promises = [];

		for (var i = 0; i < urls.length; i++) {
			var request = {
				method: 'GET',
				url: urls[i]
			};

			promises.push(call(request, true));
		}

		var promisePending = true;
		$q.all(promises).then(
			function (responses) {
				defer.resolve(responses);
			},
			function (error) {
				if (promisePending) {
					BackendResponseHandling.handleError(error);
					defer.reject(error);
					promisePending = false;
				}
			});

		return defer.promise;
	};

	// function call(request, skipErrorHandling) {
	// 	var defer = $q.defer();

	// 	AuthService.validate().then(function() {
	// 		request.url = AppInfo.ServicesURL + request.url;
	// 		if (request.url.indexOf('?') > 0) {
	// 			request.url += '&date=' + new Date().getTime();
	// 		}
	// 		else {
	// 			request.url += '?date=' + new Date().getTime();
	// 		}

	// 		$http(request).then(
	// 			function(response) {
	// 				var handledResponse = BackendResponseHandling.handle(response);
	// 				if (handledResponse) {
	// 					defer.resolve(handledResponse);
	// 				}
	// 				else {
	// 					defer.reject();
	// 				}
	// 			},
	// 			function(error) {
	// 				console.log('Error: ' + JSON.stringify(error));
	// 				if (!skipErrorHandling) BackendResponseHandling.handleError(error);
	// 				defer.reject(error);
	// 			}
	// 		);
	// 	},
	// 	function(error) {
	// 		console.log('Error: ' + JSON.stringify(error));
	// 		alert('This session has expired.');
	// 		defer.reject(error);
	// 		$state.go('login');
	// 	});

	// 	return defer.promise;
	// };


	//-- COMMON
	function call(request, skipErrorHandling) {
		request.url = AppInfo.ServicesURL + request.url;
		if (request.url.indexOf('?') > 0) {
			request.url += '&date=' + new Date().getTime();
		}
		else {
			request.url += '?date=' + new Date().getTime();
		}

		var defer = $q.defer();
		$http(request).then(
			function (response) {
				var handledResponse = BackendResponseHandling.handle(response);
				if (handledResponse != undefined) {
					//defer.resolve(DateConverterService.convertDatesToClient(handledResponse));
					defer.resolve(handledResponse);
				}
				else {
					defer.reject();
				}
			},
			function (error) {
				if (!skipErrorHandling) BackendResponseHandling.handleError(error);
				defer.reject(error);
			}
		);

		return defer.promise;
	};





};








