
angular.module('mobility-app').service('EDSAuthHttpResponseInterceptor', function($q, $injector, AppInfo) {

    /**
     * retryServiceRequest
     * @param  {[type]} config   [description]
     * @param  {[type]} deferred [description]
     * @return {[type]}          [description]
     */
    function retryServiceRequest(config, deferred) {
        function successCallback(response) {
            deferred.resolve(response);
        }

        function errorCallback(response) {
            deferred.reject(response);
        }

        var $http = $injector.get('$http');
        $http(config).then(successCallback, errorCallback);
    }

    /**
     * [retryLogin description]
     * @return {[type]} [description]
     */
    function retryLogin() {
        var def = $q.defer();

        var $localstorage = $injector.get('$localstorage');
        var AuthService = $injector.get('AuthService');

        AuthService.isAutoLoginSupported().then(
        function() {
            // check if we can re-login
            var _autoLogin = $localstorage.getAutoLogin();
            var _canReLogin = (_autoLogin == "ON");
            var currentUser = $localstorage.getLastLoggedUser();
            var _currentUser = currentUser || "";
            _canReLogin = _canReLogin && (_currentUser != "");

            // auto login   
            if (_canReLogin) {
                AuthService.validAutoLoginInfo(currentUser).then(
                function() {
                	var _params = {
                        username: currentUser,
                        autologin: true
                    };

                    AuthService.signIn(_params).then(function() {
                        def.resolve("AUTOLOGINSUCCESS");
                    });
                },
                function(err) {
                    def.reject("AUTOLOGINFAIL");
                });
            } else {
                def.reject("AUTOLOGINDISABLED");
            }
        },
        function(err) {
            def.reject("AUTOLOGINNOTSUPPORTED");
        });

        return def.promise;
    };

    var edsInterceptor = {
        responseError: function(response) {
            // check only service calls
            if (response.config.url.indexOf(AppInfo.ServicesURL) == 0) {
                if (response.status == 400 || response.status == 401) {
                    // try to reauthenticate
                    var deferred = $q.defer();

                    retryLogin().then(function(data) {
                    	// re-login worked fine, retry service request
                        retryServiceRequest(response.config, deferred);
                    }, function(err) {
                    	// we fail on login, return original response
                        deferred.reject(response);
                    });
                    return deferred.promise;
                } else {
                    return response;
                }
            } else {
                return response;
            }
        }
    };

    return edsInterceptor;

});