
angular.module('mobility-app').service('FileService', function() {
	var self = this;

	self.downloadFile = function(fileName) {
		var defer = $q.defer();

		if (!fileName || fileName.length < 1) {
			return;
		}

		if (AppInfo.useMockedFile) {
			defer.resolve('/files/' + fileName);
			return defer.promise;
		}

		var targetPath = fileName;
	    if (ionic.Platform.isAndroid()) {
	    	targetPath = cordova.file.externalCacheDirectory + fileName;
	    }
	    else {
	    	targetPath = cordova.file.documentsDirectory + fileName;
	    }

	    $ionicLoading.show();
	    new FileTransfer().download(
    		encodeURI(AppInfo.ServicesURL + '/downloads/' + fileName),
	    	targetPath,
	    	function (entry) {
    			$ionicLoading.hide();
    			defer.resolve(targetPath);
		    },
		    function (error) {
    			$ionicLoading.hide();
		    	alert('Error downloading file. ' + JSON.stringify(error));
		    	defer.reject(error);
		    },
		    true,
	    	{
		    	headers: {
					'device-type': 'browser',
					'auth-api-token': AppInfo.ApiToken,
					'auth-att-uid': $localstorage.getLastLoggedUser(),
					'Authorization': $localstorage.getBasicAuth()
		    	}
	    	}
	    );

	    return defer.promise;
	};

	self.sendFile = function(fileName) {
		cordova.plugins.email.isAvailable(function (isAvailable) {
			if (!isAvailable) {
				alert('E-mail application is not configured.');
				return;
			}

			self.downloadFile(fileName).then(function(fullpath) {
				var email = {
					to: UserService.getUserDetails().email,
					attachments: [fullpath],
					subject: AppInfo.AppName,
					body: moment().format("DD MMM YYYY - h:mm A"),
					isHtml: false
				};

				cordova.plugins.email.open(email, function () {
					// email view dismissed
				}, self);
			});
		});
		
		/*$cordovaEmailComposer.isAvailable().then(function() {
			self.downloadFile(fileName).then(function(fullpath) {
				var email = {
					app: emailApp,
					to: UserService.getUserDetails().email,
					attachments: [fullpath],
					subject: 'AT&T Mexico Dashboard',
					body: moment().format("DD MMM YYYY - h:mm A"),
					isHtml: false
				};

				$cordovaEmailComposer.open(email).then(null, function () {
					// user cancelled email
				});
			},
			function(error) {
				alert('Error creating email: ' + JSON.stringify(error));
			});
		}, function () {
			alert('Email application is not available');
			return;
		});*/
	};
});
