
angular.module('mobility-app').service('GaugeService', function($filter) {
	var self = this;

	var gauges = {};

	var COLORS = {
		'BLUE': '#027bb4',
		'GREEN': '#5ca946',
		'YELLOW': '#fcb134',
		'RED': '#dd0303',
		'GRAY': '#E7E7E7'
	};

	self.getGaugeByTitle = function(title, gaugeArr) {
		var gauge = null;
		var metric = null;

		for (var i=0; i < gaugeArr.length; i++) {
			gauge = gaugeArr[i];
			if (title != gauge.title) {
				continue;
			}

			metric = gauge.minorMetric ? gauge.minorMetric : gauge.majorMetric;
			if (metric && metric.info && metric.info.name) {
				gauge.previous = metric.info.name.trim() + ': ' + $filter('gaugeNumberFormatter')(metric.value.value, metric.info.unit);
			}
			break;
		}

		return gauge;
	};

	function updateBarThreshold(bar, gauge) {
		var interval = gauge.maxValue - gauge.minValue;
		var COLORS = {
			'green': '#5ca946',
			'yellow': '#FCB134',
			'red': '#dd0303',
			'blue': '#027bb4'
		};

		if (gauge.thresholds && gauge.thresholds.length == 2) {

			var first = Number(100 * (gauge.thresholds[0].value - gauge.minValue) / interval).toFixed(0);
			var second = Number(100 * (gauge.thresholds[1].value - gauge.minValue) / interval).toFixed(0);

			var startColor = COLORS[gauge.startWithStatus.toLowerCase()];
			var firstColor = COLORS[gauge.thresholds[0].status.toLowerCase()];
			var secondColor = COLORS[gauge.thresholds[1].status.toLowerCase()];

			var gradient = 'left, ' + startColor + ', ' + startColor + ' ' + first + '%, ' + firstColor + ' ' + first + '%, ' + firstColor + ' ' + second + '%, ' + secondColor + ' ' + second + '%';
			bar.style.background = '-webkit-linear-gradient(' + gradient + ')';
			bar.style.background = '-moz-linear-gradient(' + gradient + ')';
			bar.style.background = '-ms-linear-gradient(' + gradient + ')';
			bar.style.background = 'linear-gradient(' + gradient + ')';
		}
		else {
			bar.style.background = COLORS[gauge.startWithStatus.toLowerCase()];
		}
	};

	self.drawHorizontalGauge = function(bar, gauge) {
		var transformation = "";

		updateBarThreshold(bar, gauge);

		var percentage = (gauge.metric.value.value - gauge.minValue) / (gauge.maxValue - gauge.minValue);
		if (percentage > 1) percentage = 1;
		if (percentage < 0) percentage = 0;

		var position = Math.round(bar.offsetWidth * percentage);

		var marker = bar.childNodes[1];
		position = position - Math.round(marker.offsetWidth/2);
		if (position < 0) position = 0;

		transformation = "translate(" + position + "px, -50%)";
		marker.style.webkitTransform = transformation;
		marker.style.MozTransform = transformation;
		marker.style.msTransform = transformation;
		marker.style.transform = transformation;
	};

	self.draw = function(elementId, gauge, type) {
		var unit = gauge.metric.info.unit;

		var data = {
			min: gauge.minValue,
			max: gauge.maxValue,
			label: {
				min: $filter('gaugeNumberFormatter')(gauge.minValue, unit, true),
				max: $filter('gaugeNumberFormatter')(gauge.maxValue, unit, true),
				value: $filter('gaugeNumberFormatter')(gauge.metric.value.value, unit, false)
			},
			value: gauge.metric.value.value,
			direction: gauge.direction,
			color: COLORS[gauge.metric.value.status]
		};

		var options = { gauge: {} };
		if (gauge.thresholds && gauge.thresholds.length > 0) {
			options.gauge.type = 'threshold';
			options.gauge.threshold = {
				type: 'value',
				limits: []
			};

			// first value
			options.gauge.threshold.limits.push({
				start: gauge.minValue,
				end: gauge.thresholds[0].value,
				color: COLORS[gauge.startWithStatus]
			});

			for (var i=0; i < gauge.thresholds.length; i++) {
				// if threshold value is out of boundaries, use single gauge
				if (gauge.thresholds[i].value < gauge.minValue || gauge.thresholds[i].value > gauge.maxValue || (0 == gauge.thresholds.length-1 && gauge.thresholds[i].value == gauge.maxValue)) {
					options = { gauge: {} };
					break;
				}

				options.gauge.threshold.limits.push({
					start: gauge.thresholds[i].value,
					end: (i == gauge.thresholds.length-1) ? gauge.maxValue : gauge.thresholds[i+1].value,
					color: COLORS[gauge.thresholds[i].status]
				});
			}
		}
		else if (type == 'arrow') {
			if (gauge.metric.value.value > gauge.minValue && gauge.metric.value.value < gauge.maxValue) {
				options.gauge.type = 'threshold';
				options.gauge.threshold = {
					type: 'value',
					limits: []
				};

				// first value
				options.gauge.threshold.limits.push({
					start: gauge.minValue,
					end: gauge.metric.value.value,
					color: COLORS[gauge.startWithStatus]
				});

				// second value
				options.gauge.threshold.limits.push({
					start: gauge.metric.value.value,
					end: gauge.maxValue,
					color: COLORS['GRAY']
				});
			}
			else if (gauge.metric.value.value < gauge.minValue) {
				options.gauge.type = 'single';
				options.gauge.single = { color: COLORS['GRAY'] };
			}
		}

		if (type) {
			options.tick = { type: type };
		}

		if (!options.gauge.type) {
			options.gauge.type = 'single';
			options.gauge.single = { color: COLORS[gauge.startWithStatus] };
		}

		if (gauges[elementId]) {
			gauges[elementId].destroy();
		}

		gauges[elementId] = new Gauge().create(elementId, data, options);
		return gauges[elementId];
	};
});
