
angular.module('mobility-app').service('GraphService', function ($filter) {
	var self = this;

	var Y_VALUE_WHEN_NULL = null;
	var PLOT_BAND_COLOR = '#f2f2f2';
	var APP_MIN_COLOR = 'green';
	var APP_MAX_COLOR = 'red';


	var graphs = {};

	var phoneView = (screen.width < 768);

	//-- GRAPH FUNCTIONS
	self.drawGraph = function (id, data) {
		//if (graphs[id] && graphs[id].destroy) graphs[id].destroy();
		graphs[id] = new Highcharts.Chart(graphConfig(id, data));
		return graphs[id];
	};



	self.getGraphDataByTitle = function(title, graphDataArr) {
		var graphData = null;
		
		for (var i=0; i < graphDataArr.length; i++) {
			graphData = graphDataArr[i];
			if (title != graphData.title) {
				continue;
			}

		
			break;
		}

		return graphData;
	};





	self.drawGraphWithVariant = function (id, data) {
		graphs[id] = new Highcharts.Chart(variantGraphConfig(id, data));
		return graphs[id];
	};


	self.drawGraphWithLegend = function (id, data, position, visibility, multiAxis, category) {
		//if (graphs[id] && graphs[id].destroy) graphs[id].destroy();
		graphs[id] = new Highcharts.Chart(graphConfigWithLegend(id, data, visibility, multiAxis, category));
		drawNotes(id, graphs[id], data);
		self.highlightArea(id, position);
		return graphs[id];
	};

	self.drawBarGraph = function (id, data) {
		graphs[id] = new Highcharts.Chart(barGraphConfig(id, data));
		return graphs[id];
	};

	self.drawPieChart = function (id, data) {
		graphs[id] = new Highcharts.Chart(pieChartConfig(id, data));
		return graphs[id];
	};

	// --- CALL BACKS - START
	var graphClickEventCallBack;
	function notifyGraphClickEvent(index) {
		if (graphClickEventCallBack) graphClickEventCallBack(index);
	};
	self.registerGraphClickEventCallBack = function (callBack) {
		graphClickEventCallBack = callBack;
	};

	var noteClickEventCallBack;
	function notifyNoteClickEvent(index, xPos, yPos) {
		if (noteClickEventCallBack) noteClickEventCallBack(index, xPos, yPos);
	};
	self.registerNoteClickEventCallBack = function (callBack) {
		noteClickEventCallBack = callBack;
	};
	// --- CALL BACKS - END

	self.getColorForSerie = function (chartId, serieIndex) {
		var chart = $('#' + chartId).highcharts();
		if (!chart) {
			return 'black';
		}

		return chart.get('serie' + serieIndex).data[0].color;
	};

	self.setSerieVisibility = function (chartId, serieIndex, visibility) {
		var serie = $('#' + chartId).highcharts().get('serie' + serieIndex);
		if (visibility) {
			serie.show();
		} else {
			serie.hide();
		}
	};

	var highlightedPosition = 0;
	self.highlightArea = function (chartId, position) {
		var axis = graphs[chartId].xAxis[0];
		removeAllPlotBands(axis);
		addPlotBandAtPosition(axis, position);
		highlightedPosition = position;
	};

	function getPositionForNote(note, data) {
		for (var i = 0; i < data.metricSeries.length; i++) {
			for (var j = 0; j < data.metricSeries[i].values.length; j++) {
				if (data.metricSeries[i].values[j].date == note.date) {
					return j;
				}
			}
		}
		return -1;
	};

	function showNotification(id, event) {
		var pos = event.target.attributes.position.value;
		var x = graphs[id].series[0].points[pos].plotX + graphs[id].series[0].xAxis.left;
		var y = graphs[id].series[0].yAxis.top;

		var values = {
			chart: event.target.attributes.chart.value,
			position: pos,
			index: event.target.attributes.index.value,
			left: x,
			top: y,
			offsetLeft: graphs[id].series[0].xAxis.left,
			offsetRight: graphs[id].series[0].xAxis.right,
			pointer: graphs[id].series[0].points[0].plotX
		};

		event.preventDefault();
		event.stopPropagation();
		notifyNoteClickEvent(values);
	};

	function drawNotes(id, graph, data) {
		var notes = data.notes;
		if (phoneView || !graph || !notes || notes.length < 1) return;

		var renderer = graph.renderer;
		if (!renderer) return;

		// draw the notes marker positions
		var x, y, pos = 0;
		var circleRadius = 15;
		for (var i = 0; i < notes.length; i++) {
			pos = getPositionForNote(notes[i], data);
			if (pos < 0) continue;

			x = graph.series[0].points[pos].plotX + graph.series[0].xAxis.left;
			y = graph.series[0].yAxis.top - (circleRadius * 2);
			renderer.circle(x, y, circleRadius)
				.attr({
					'id': id + '-clickable-note-marker-' + i,
					fill: 'transparent',
					zIndex: 25,
					'stroke-width': 0,
					index: i,
					position: pos,
					chart: id
				})
				.on('click', function (event) { showNotification(id, event) })
				.add();

			renderer.circle(x, y, circleRadius / 3)
				.attr({
					'id': id + '-note-marker-' + i,
					'class': 'note-marker',
					zIndex: 22
				}).add();
		}
	};

	function getXAxisData(data, format) {
		var values = [];

		var outputFormat = format;

		var _shiftHour = false;

		if (!outputFormat || outputFormat.length < 1) {
			if (data.metricSeries[0].info.updateSchedule == 'HOURLY') {
				outputFormat = 'HH';
				_shiftHour = true; // server value starts from 0 , we need to shift it one hour
			} else if (data.metricSeries[0].info.updateSchedule == 'DAILY') {
				outputFormat = 'DD-MMM';
			} else if (data.metricSeries[0].info.updateSchedule == 'MONTHLY') {
				outputFormat = 'MMM';
			} else {
				outputFormat = 'YYYY-MM-DD';
			}
		}

		for (var i = 0; i < data.metricSeries[0].values.length; i++) {


			var _labelStr = $filter('dateFormatter')(data.metricSeries[0].values[i].date, outputFormat);

			try {
				if (_shiftHour) {
					_labelStr = Number.parseInt(_labelStr) + 1;
				}
			} catch (err) {

			}


			values.push(_labelStr);
		};

		return values;
	};

	function getYAxis(data, unit, multiAxis) {
		if (multiAxis) {
			var unitLocal = unit;
			var axis = [];
			for (var i = 0; i < data.metricSeries.length; i++) {
				unitLocal = data.metricSeries[i].info.unit;
				axis.push({
					allowDecimals: false,
					alternateGridColor: '#ffffff',
					gridLineColor: '#E7E7E7',
					lineColor: '#E7E7E7',
					tickColor: '#E7E7E7',
					title: { text: '' },
					tickInterval: (unitLocal == 'MINUTES' ? 15 : undefined),
					labels: {
						formatter: function () {
							return formatYAxisValue(this.value, unitLocal);
						}
					},
					opposite: (i > 0)
				});
			}
			return axis;
		}

		return {
			allowDecimals: false,
			alternateGridColor: '#ffffff',
			gridLineColor: '#E7E7E7',
			lineColor: '#E7E7E7',
			tickColor: '#E7E7E7',
			title: { text: '' },
			//max: (unit == 'PERCENTAGE' ? 100 : undefined),
			tickInterval: (unit == 'MINUTES' ? 15 : undefined),
			labels: {
				formatter: function () {
					return formatYAxisValue(this.value, unit);
				}
			}
		};
	};

	function getYAxisData(data, visibility, multiAxis, category) {
		var values = [];
		var y;

		if (Array.isArray(data)) {
			var index = 0;
			for (var i = 0; i < data.length; i++) {
				for (var j = 0; j < data[i].metricSeries.length; j++) {
					if (category && data[i].metricSeries[j].info.category != category) {
						continue;
					}

					values[index] = {
						id: 'serie' + i,
						name: data[i].metricSeries[j].info.name,
						zIndex: data[i].metricSeries.length - i,
						visible: (visibility != undefined && visibility[i] != undefined) ? visibility[i] : true,
						data: []
					};

					for (var k = 0; k < data[i].metricSeries[j].values.length; k++) {
						y = data[i].metricSeries[j].values[k].value;
						values[index].data.push(y ? y : Y_VALUE_WHEN_NULL);
					}

					index++;
				}
			}
		}

		else {
			var index = 0;
			for (var i = 0; i < data.metricSeries.length; i++) {

				if (category && data.metricSeries[i].info.category != category) {
					continue;
				}

				values[index] = {
					id: 'serie' + i,
					name: data.metricSeries[i].info.name,
					yAxis: multiAxis ? i : 0,
					zIndex: data.metricSeries.length - i,
					visible: (visibility != undefined && visibility[i] != undefined) ? visibility[i] : true,
					data: []
				};

				for (var j = 0; j < data.metricSeries[i].values.length; j++) {
					y = data.metricSeries[i].values[j].value;
					values[index].data.push((y || y === 0) ? y : Y_VALUE_WHEN_NULL);
				}

				index++;
			}
		}

		return values;
	};

	function formatYAxisValue(val, unit) {
		//return $filter('graphNumberFormatter')(val, unit, true);

		return $filter('graphNumberFormatter')(val, unit, false);


	};

	function removeAllPlotBands(axis) {
		for (var i = axis.plotLinesAndBands.length - 1; i >= 0; i--) {
			axis.removePlotBand(axis.plotLinesAndBands[i].id);
		}
	};

	function addPlotBandAtPosition(axis, pos) {
		axis.addPlotBand(getPlotBandAtPosition(pos));
	};

	function getPlotBandAtPosition(pos) {
		return {
			color: PLOT_BAND_COLOR,
			from: pos - 0.5,
			to: pos + 0.5,
			id: 'plotBand' + pos
		};
	};

	function graphConfigWithLegend(id, data, visibility, multiAxis, category) {
		var unit = data.metricSeries[0].info.unit;

		var graphObj = {
			chart: {
				marginTop: 70,
				renderTo: id,
				events: {
					click: function (e) {
						var pos = Math.abs(e.xAxis[0].value);
						if (pos - Math.floor(pos) > 0.5) {
							pos = Math.floor(pos) + 1;
						} else {
							pos = Math.floor(pos);
						}

						// only highlight the area of the graph that has data
						for (var i = 0; i < e.yAxis[0].axis.series.length; i++) {
							if (e.yAxis[0].axis.series[i].points[pos].plotY !== undefined) {
								//removeAllPlotBands(e.xAxis[0].axis);
								//addPlotBandAtPosition(e.xAxis[0].axis, pos);
								notifyGraphClickEvent(pos);
							}
						}
					}
				}
			},
			credits: {
				enabled: false
			},
			title: {
				text: ''
			},
			subtitle: {
				text: ''
			},
			legend: {
				enabled: false
			},
			xAxis: {
				categories: getXAxisData(data),
				lineColor: '#E7E7E7',
				gridLineColor: '#E7E7E7',
				tickColor: '#E7E7E7'
			},
			yAxis: getYAxis(data, unit, multiAxis),
			plotOptions: {
				series: {
					animation: false,
					enableMouseTracking: false,
					marker: {
						enabled: true,
						fillColor: phoneView ? null : 'white',
						lineColor: phoneView ? null : undefined,
						lineWidth: phoneView ? 0 : 2,
						radius: phoneView ? 3 : 4,
						symbol: 'circle'
					},
					lineWidth: 2
				}
			},
			series: getYAxisData(data, visibility, multiAxis, category)
		};

		return addYMinMaxPlotLines(graphObj,data);
	};



	function addYMinMaxPlotLines(graphObj, data) {

		if (data.minMaxValue) {
			// calc max Y
			var _allVal = [];
			graphObj.series.forEach(function (seriVal) {
				seriVal.data.forEach(function (val) {
					if (val != null) {
						_allVal.push(val);
					}
				});
			});

			_allVal.push(data.minMaxValue.min.value || 0);
			_allVal.push(data.minMaxValue.max.value || 0);

			var max = _allVal.reduce(function (a, b) {
				return Math.max(a, b);
			});

			graphObj.yAxis.max = max * 1.1;
			graphObj.yAxis.plotLines = graphObj.yAxis.plotLines || [];
			var _plotLineZIndex = (graphObj.series.length || 0) + 1;


			graphObj.yAxis.plotLines.push({
					value: data.minMaxValue.min.value,
					color: APP_MIN_COLOR,
					width: 2,
					label: {
						text: data.minMaxValue.min.text,
						align: 'left',
						style: {
							color: APP_MIN_COLOR
						}
					},
					zIndex: _plotLineZIndex
			});

			graphObj.yAxis.plotLines.push({
					value: data.minMaxValue.max.value,
					color: APP_MAX_COLOR,
					width: 2,
					label: {
						text: data.minMaxValue.max.text,
						align: 'left',
						style: {
							color: APP_MAX_COLOR
						}
					},
					zIndex: _plotLineZIndex
			});



			// if (!graphObj.legend.enabled){
			// 	graphObj.legend = {
			// 		enabled: true
			// 	};
			// }





		}


		return graphObj;

	};


	function graphConfig(id, data) {
		var graphObj = {
			chart: {
				marginTop: 70,
				renderTo: id
			},
			credits: {
				enabled: false
			},
			title: {
				text: ''
			},
			subtitle: {
				text: ''
			},
			legend: {
				enabled: data.metricSeries.length > 1,
				align: phoneView ? 'left' : 'right',
				verticalAlign: 'top',
				x: -7,
				y: phoneView ? 10 : 20,
				symbolWidth: 10,
				itemDistance: phoneView ? 10 : 20,
				itemStyle: {
					"fontSize": "11px",
					"fontWeight": "normal"
				}
			},
			xAxis: {
				categories: getXAxisData(data),
				lineColor: '#E7E7E7',
				gridLineColor: '#E7E7E7',
				tickColor: '#E7E7E7'
			},
			yAxis: {
				alternateGridColor: '#ffffff',
				gridLineColor: '#E7E7E7',
				lineColor: '#E7E7E7',
				tickColor: '#E7E7E7',
				title: { text: '' },
				labels: {
					formatter: function () {
						return formatYAxisValue(this.value);
					}
				}
			},
			plotOptions: {
				series: {
					animation: false,
					enableMouseTracking: false,
					marker: {
						enabled: true,
						fillColor: 'white',
						lineColor: undefined,
						lineWidth: 2,
						symbol: 'circle'
					},
					lineWidth: 2,
					events: {
						legendItemClick: function () {
							return false;
						}
					}
				}
			},
			series: getYAxisData(data)
		};


		return addYMinMaxPlotLines(graphObj,data);

		
	};

	function getBarXAxisData(data) {
		var names = [];
		var str;
		for (var i = 0; i < data.contributors.length; i++) {
			str = data.contributors[i].activity;
    		/*if (str.length > 21) {
    			str = str.substring(0, 21) + '...';
    		}*/
			names.push(str);
		}

		return names;
	};

	function getBarYAxisData(data) {
		var series = [{
			data: []
		}];

		for (var i = 0; i < data.contributors.length; i++) {

			series[0].data.push(data.contributors[i].amount);
		}

		return series;
	};

	function barGraphConfig(id, data) {
		var graphObj = {
			chart: {
				type: 'bar',
				renderTo: id
			},
			credits: {
				enabled: false
			},
			title: {
				text: ''
			},
			subtitle: {
				text: ''
			},
			legend: {
				enabled: false
			},
			xAxis: {
				categories: getBarXAxisData(data)
			},
			yAxis: {
				min: 0,
				lineColor: '#E7E7E7',
				lineWidth: 1,
				gridLineWidth: 0,
				title: {
					text: ''
				}
			},
			plotOptions: {
				bar: {
					enableMouseTracking: false,
					borderRadius: 5,
					maxPointWidth: 15,
					colorByPoint: true,
					dataLabels: { // To customize offers value
						enabled: true,
						style: {
                    		fontWeight: 'normal'
                		},
						formatter: function () {
							return formatYAxisValue(this.y);
						}
					}
				}
			},
			series: getBarYAxisData(data)
		};

		return graphObj;
	};

	function getPieChartData(data) {
		var series = [{
			colorByPoint: true,
			data: []
		}];

		for (var i = 0; i < data.length; i++) {
			series[0].data.push({
				name: data[i].name,
				y: data[i].value
			});
		}

		return series;
	};

	function pieChartConfig(id, data) {
		var graphObj = {
			chart: {
				plotBackgroundColor: null,
				plotBorderWidth: null,
				plotShadow: false,
				type: 'pie',
				renderTo: id
			},
			colors: ['#022a5b', '#1e5583', '#29abe2', '#7dd1ef', '#ec1c24'],
			credits: {
				enabled: false
			},
			title: {
				text: ''
			},
			subtitle: {
				text: ''
			},
			tooltip: {
				enabled: false
			},
			plotOptions: {
				pie: {
					allowPointSelect: false,
					cursor: 'pointer',
					dataLabels: {
						enabled: true,
						formatter: function () {
							return formatYAxisValue(this.point.y, 'VALUE') + ' (' + this.percentage.toFixed(2) + ' %)';
						},
						style: {
							color: (Highcharts.theme && Highcharts.theme.contrastTextColor) || 'black'
						}
					},
					showInLegend: true,
					point: {
						events: {
							legendItemClick: function () {
								return false;
							}
						}
					}
				}
			},
			series: getPieChartData(data)
		};

		return graphObj;
	};



	function getYAxisDataForVariantGraph(data, visibility, multiAxis, category) {
		var values = [];
		var y;

		var _chartType = 'spline';
		var _areaChartType = 'areasplinerange';

		if (Array.isArray(data)) {
			var index = 0;
			var isAreaData = false;
			for (var i = 0; i < data.length; i++) {
				for (var j = 0; j < data[i].metricSeries.length; j++) {
					if (category && data[i].metricSeries[j].info.category != category) {
						continue;
					}

					values[index] = {
						id: 'serie' + i,
						type: _chartType,
						name: data[i].metricSeries[j].info.name,
						// color: Highcharts.getOptions().colors[i],
						zIndex: data[i].metricSeries.length - i,
						visible: (visibility != undefined && visibility[i] != undefined) ? visibility[i] : true,
						data: []
					};

					isAreaData = false;

					for (var k = 0; k < data[i].metricSeries[j].values.length; k++) {
						y = data[i].metricSeries[j].values[k].value;

						isAreaData = isAreaData | Array.isArray(y); // we may have some entries without 2 entries


						values[index].data.push(y ? y : Y_VALUE_WHEN_NULL);
					}

					if (isAreaData == true) {
						values[index].type = _areaChartType;
						values[index].fillOpacity = 0.8;
						values[index].color = '#EFEFEF';
						values[index].lineWidth = 0;
						values[index].showInLegend = false;
					}


					index++;
				}
			}
		}

		else {
			var index = 0;
			for (var i = 0; i < data.metricSeries.length; i++) {

				if (category && data.metricSeries[i].info.category != category) {
					continue;
				}

				values[index] = {
					id: 'serie' + i,
					type: _chartType,
					name: data.metricSeries[i].info.name,
					yAxis: multiAxis ? i : 0,
					zIndex: data.metricSeries.length - i,
					// color: Highcharts.getOptions().colors[i],
					visible: (visibility != undefined && visibility[i] != undefined) ? visibility[i] : true,
					data: []
				};

				for (var j = 0; j < data.metricSeries[i].values.length; j++) {
					y = data.metricSeries[i].values[j].value;

					isAreaData = isAreaData | Array.isArray(y); // we may have some entries without 2 entries

					values[index].data.push((y || y === 0) ? y : Y_VALUE_WHEN_NULL);
				}


				if (isAreaData == 1) {
					values[index].type = _areaChartType;
					values[index].fillOpacity = 0.8;
					values[index].color = '#EFEFEF';
					values[index].lineWidth = 0;
					values[index].showInLegend = false;
				}




				index++;
			}
		}

		return values;
	};


	function variantGraphConfig(id, data) {
		var graphObj = graphConfig(id, data);
		// repopulate with variant data

		graphObj.colors = ['#7cb5ec', APP_BAR_COLOR];
		graphObj.series = getYAxisDataForVariantGraph(data);
		return graphObj;
	};











});

