angular.module('mobility-app').factory('$localstorage', ['$window', function ($window) {

	function set(key, value) {
		$window.localStorage[key] = value;
	};

	function get(key, defaultValue) {
		return $window.localStorage[key] || defaultValue;
	};

	function clear(key) {
		if (key) {
			delete $window.localStorage[key];
		}
		else {
			$window.localStorage.clear();
		}
	};

	return {
		set: set,
		get: get,
		clear: clear,

		setObject: function (key, value) {
			set(key, JSON.stringify(value));
		},
		getObject: function (key) {
			return JSON.parse(get(key, '{}'));
		},

		setLastLoggedUser: function (value) {
			set('user', value);
		},
		getLastLoggedUser: function () {
			return get('user', '');
		},

		setLastLoggedUser: function (value) {
			set('user', value);
		},
		getLastLoggedUser: function () {
			return get('user', '');
		},

		setAuthorization: function (auth) {
			set('Authorization', auth);
		},
		getAuthorization: function () {
			return get('Authorization');
		},
		removeAuthorization: function () {
			clear('Authorization');
		},

		setAutoLogin: function (enabled) {
			set('autologin', enabled);
		},
		getAutoLogin: function () {
			return get('autologin', '');
		},

		setTokenExpiration: function (expirationInSeconds) {
			var expirationTime = moment().add(expirationInSeconds, 's').format('YYYYMMDDHHmmss');
			set('TokenExpiration', expirationTime);
		},
		isTokenExpired: function () {
			var expirationTime = get('TokenExpiration');
			if (!expirationTime) {
				return true;
			}

			return moment().isAfter(moment(expirationTime, 'YYYYMMDDHHmmss'));
		},
		setLastGlossaryLoadDate: function (value) {
			set('lastGlossaryLoadDate', value);
		},
		getLastGlossaryLoadDate: function () {
			var lastLoadedDate = get('lastGlossaryLoadDate');
			if (!lastLoadedDate) {
				return 0;
			}

			return lastLoadedDate;
		}
	}
}]);
